<?php

class dmeng_Options_Default {
  
  public $tabs;

  public function __construct() {
    
    $this->tabs = array(
      'main' => __('概况', 'dmeng'),
      'general' => __('常规设置', 'dmeng'),
      'home' => __('首页', 'dmeng'),
      'writing' =>__('撰写','dmeng'),
      'reading' =>__('阅读','dmeng'),
      'adsense' =>__('广告','dmeng'),
      'discussion' =>__('讨论','dmeng'),
      'open' =>__('开放平台','dmeng'),
      'credit' =>__('积分','dmeng'),
      'gift' =>__('积分换礼','dmeng'),
      'smtp' =>__('SMTP','dmeng'),
      'seo' =>__('SEO','dmeng'),
      'mobile' =>__('移动端','dmeng')
    );

    add_filter('dmeng_options_page-tabs', array($this, 'add_tab'));

    foreach (array_keys($this->tabs) as $key) {
      add_action('dmeng_options_page_'.$key, array($this, $key.'_options'));
    }

  }

  public function add_tab($tabs) {
    $tabs = array_merge($this->tabs, $tabs);
    return $tabs;
  }
  
  public function main_options() {
  ?>
  <div class="options-box">
    <span style="color:#a94442;background:#f2dede;padding:0 5px"><?php _e('请注意：此版本为测试版，部分功能不可用，不带有主题升级检查功能，请保持关注并及时获取最新版本！', 'dmeng');?></span>

    <h3><?php _e('统计', 'dmeng');?></h3>

    <?php
      $label = array(
        'post' => __( '文章', 'dmeng' ),
        'cat' => __( '分类', 'dmeng' ),
        'tag' => __( '标签', 'dmeng' ),
        'user' => __( '用户', 'dmeng' ),
        'comment' => __( '评论', 'dmeng' ),
        'view' => __( '浏览总数', 'dmeng' ),
        'search' => __( '搜索次数', 'dmeng' ),
      );
      global $dmeng_Count;
      foreach ($label as $type=>$title) {
        $count = $dmeng_Count->get($type);
        if ( !empty($count) )
          echo ''.$title.' <code>'.$count.'</code> ';
      }
    ?>

    <h3><?php _e('链接', 'dmeng');?></h3>
    <ul>
      <li>主题官网：<a href="http://www.dmeng.net/" target="_blank">http://www.dmeng.net/</a></li>
      <li>作者主页：<a href="http://duomeng.me/" target="_blank">http://duomeng.me/</a></li>
    </ul>

    <h3><?php _e('致谢', 'dmeng');?></h3>
    <p>感谢以下小伙伴提供的支持</p>
    <ol>
      <li>Bootstrap <a href="http://getbootstrap.com/" target="_blank">http://getbootstrap.com/</a></li>
      <li>jQuery <a href="http://jquery.com/" target="_blank">http://jquery.com/</a></li>
      <li>Lazy Load Plugin for jQuery <a href="http://www.appelsiini.net/projects/lazyload" target="_blank">http://www.appelsiini.net/</a></li>
      <li>iScroll <a href="http://cubiq.org/iscroll-5" target="_blank">http://cubiq.org/iscroll-5</a></li>
      <li>七牛云存储 <a href="https://portal.qiniu.com/signup?code=3ldifmmzc22qa" target="_blank">https://www.qiniu.com/</a></li>
      <li>阿里云ECS <a href="http://www.aliyun.com/product/ecs/?ali_trackid=2:mm_46071101_7414056_29420481:1421314877_2k2_2067429985" target="_blank">http://www.aliyun.com/</a></li>
    </ol>

  </div>
    
  <?php
  }

  public function general_options() {

    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );

      update_option('zh_cn_l10n_icp_num', sanitize_text_field($data['zh_cn_l10n_icp_num']));

      update_option('dmeng_general_setting', json_encode(array(
        'head_code' => $data['head_code'],
        'footer_code' => $data['footer_code'],
        'head_css' => $data['head_css']
      )));
      
      update_option('dmeng_black_list', $data['black_list']);

      echo dmeng_options_notice();
    }

    $general = (array)wp_unslash(json_decode(get_option('dmeng_general_setting'), true));

    $head_code = empty($general['head_code']) ? '' : $general['head_code'];
    $footer_code = empty($general['footer_code']) ? '' : $general['footer_code'];
    $head_css = empty($general['head_css']) ? '' : $general['head_css'];

    $rows = array();
    
    $rows[] = array(
      'type' => 'input',
      'name' => 'zh_cn_l10n_icp_num',
      'value' => get_option('zh_cn_l10n_icp_num'),
      'title' => __('ICP备案号', 'dmeng'),
      'description' => __('会被（工信部网站）链接包含并显示在网站底部。', 'dmeng')
    );   
    
    $rows[] = array(
      'type' => 'textarea',
      'classes' => 'code',
      'name' => 'head_code',
      'value' => $head_code,
      'title' => __('头部HEAD代码', 'dmeng'),
      'description' => __('如添加meta信息验证网站所有权。', 'dmeng')
    );   
    
    $rows[] = array(
      'type' => 'textarea',
      'classes' => 'code',
      'name' => 'footer_code',
      'value' => $footer_code,
      'title' => __('脚部统计代码', 'dmeng'),
      'description' => __('放置统计代码或安全网站认证小图标等。', 'dmeng')
    );   
    
    $rows[] = array(
      'type' => 'textarea',
      'classes' => 'code',
      'name' => 'head_css',
      'value' => $head_css,
      'title' => __('自定义CSS', 'dmeng'),
      'description' => __('以下内容会被放置在&lt;style&gt;标签之内，无需输入&lt;style type="text/css"&gt;和&lt;/style&gt;。', 'dmeng')
    );    
    
    $rows[] = array(
      'type' => 'textarea',
      'classes' => 'code',
      'name' => 'black_list',
      'value' => get_option('dmeng_black_list'),
      'title' => __('登录安全', 'dmeng'),
      'description' => __('请使用 | 分隔开，而且 | 前后都不要留空格。请谨慎操作！黑名单里的用户名都不能用来注册，也不能登录。', 'dmeng')
    );

    echo dmeng_options_form_output(
      array(
        'method' => 'post',
        'class' => 'options-box'
      ),
      dmeng_options_new_nonce(__FUNCTION__) . dmeng_options_table_build($rows)
    );

  }

  public function home_options() {

    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );

      update_option('dmeng_home_seo', json_encode(array(
        'keywords' => $data['keywords'],
        'description' => $data['description']
      )));

      echo dmeng_options_notice();
    }

    $general = (array)wp_unslash(json_decode(get_option('dmeng_home_seo'), true));
    
    $keywords = empty($general['keywords']) ? '' : $general['keywords'];
    $description = empty($general['description']) ? '' : $general['description'];
    
    $rows = array();

    $rows[] = array(
      'type' => 'textarea',
      'name' => 'keywords',
      'value' => $keywords,
      'title' => __('首页关键词', 'dmeng'),
      'description' => __('网站首页的网页关键词。', 'dmeng')
    );

    $rows[] = array(
      'type' => 'textarea',
      'name' => 'description',
      'value' => $description,
      'title' => __('首页描述', 'dmeng'),
      'description' => __('网站首页的网页描述，推荐200字以内。', 'dmeng')
    );

    echo dmeng_options_form_output(
      array(
        'method' => 'post',
        'class' => 'options-box'
      ),
      dmeng_options_new_nonce(__FUNCTION__) . dmeng_options_table_build($rows)
    );

  }

  public function writing_options() {

    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );
      
      if (isset($data['can_post_cat']))
        update_option('dmeng_can_post_cat', json_encode(array_map('intval', $data['can_post_cat'])));

      if (isset($data['copyright_content_default']))
        update_option('dmeng_copyright_content_default', sanitize_text_field($data['copyright_content_default']));        

      foreach (array('copyright_status_default', 'post_index', 'post_min_strlen', 'post_max_strlen') as $key) {
        if (isset($data[$key]))
          update_option('dmeng_'.$key, (int)$data[$key]);
      }

      echo dmeng_options_notice();
    }
    
    $copyright_status_default = (int)get_option('dmeng_copyright_status_default', 1);
    $copyright_content_default = wp_unslash(get_option('dmeng_copyright_content_default'));
    $post_index = (int)get_option('dmeng_post_index', 1);
  
    $can_post_cat = array_map('intval', (array)wp_unslash(json_decode(get_option('dmeng_can_post_cat'), true)));
    
    $categories_array = array();
    $categories = get_categories( array('hide_empty' => 0) );
    foreach ( $categories as $category ) {
      $categories_array[$category->term_id] = $category->name;
    }

    $rows = array();

    $rows[] = array(
      'type' => 'select',
      'name' => 'copyright_status_default',
      'value' => $copyright_status_default,
      'options' => array(
        1 => __( '显示', 'dmeng' ),
        0 => __( '不显示', 'dmeng' )
      ),
      'title' => __('默认版权声明开关', 'dmeng'),
      'description' => __('在文章/页面内容下的版权声明', 'dmeng')
    );
    
    $rows[] = array(
      'type' => 'textarea',
      'name' => 'copyright_content_default',
      'value' => $copyright_content_default,
      'title' => __('默认版权声明内容', 'dmeng'),
      'description' => __('版权声明内容，文章链接用{link}表示，文章标题用{title}表示，站点地址用{url}表示，站点名称用{name}表示', 'dmeng')
    );
    
    $rows[] = array(
      'type' => 'select',
      'name' => 'post_index',
      'value' => $post_index,
      'options' => array(
        1 => __( '显示', 'dmeng' ),
        0 => __( '不显示', 'dmeng' )
      ),
      'title' => __('默认锚点导航开关', 'dmeng'),
      'description' => __('选择是时将把文章页和页面内容中的H标题生成锚点导航目录。', 'dmeng')
    );

    
    $post_rows = array();

    $post_rows[] = array(
      'type' => 'checkbox',
      'name' => 'can_post_cat',
      'value' => $can_post_cat,
      'options' => $categories_array,
      'title' => __('允许投稿的分类', 'dmeng'),
      'description' => __('不选择任何分类则不开放投稿', 'dmeng')
    );
    
    $post_rows[] = array(
      'type' => 'input',
      'name' => 'post_min_strlen',
      'value' => get_option('dmeng_post_min_strlen', 140),
      'title' => __('投稿的最少字数', 'dmeng'),
      'description' => __('限制最少字数，一篇文章少于这个字数不允许投稿。一条微博是140，所以默认是140字。', 'dmeng')
    );
    
    $post_rows[] = array(
      'type' => 'input',
      'name' => 'post_max_strlen',
      'value' => get_option('dmeng_post_max_strlen', 12000),
      'title' => __('投稿的最多字数', 'dmeng'),
      'description' => __('限制最多字数，一篇文章超过这个字数不允许投稿。刊物文章一般是2200-12000，所以默认12000字。', 'dmeng')
    );


    echo dmeng_options_form_output(
      array(
        'method' => 'post',
        'class' => 'options-box'
      ),
      dmeng_options_new_nonce(__FUNCTION__).
      dmeng_options_table_build($rows).
      '<h3>'.__('投稿','dmeng').'</h3>'.
      dmeng_options_table_build($post_rows)
    );
    
  }
  
  public function reading_options() {

    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );

      foreach (array('copyright_status_all', 'post_index_all', 'google_code_prettify') as $key) {
        if (isset($data[$key]))
          update_option('dmeng_'.$key, (int)$data[$key]);
      }
      
      if (isset($data['post_thumbnail']))
        update_option('dmeng_post_thumbnail', json_encode($data['post_thumbnail']));
      
      echo dmeng_options_notice();
    }

    $copyright_status_all = (int)get_option('dmeng_copyright_status_all', 1);
    $post_index_all = (int)get_option('dmeng_post_index_all', 1);
    $post_thumbnail = (array)wp_unslash(json_decode(get_option('dmeng_post_thumbnail','{"on":"1","suffix":"?imageView2/1/w/220/h/146/q/100"}'), true));

    $google_code_prettify = (int)get_option('dmeng_google_code_prettify', 0);

    $rows = array();

    $rows[] = array(
      'type' => 'select',
      'name' => 'copyright_status_all',
      'value' => $copyright_status_all,
      'options' => array(
        1 => __('显示', 'dmeng'),
        0 => __('不显示', 'dmeng')
      ),
      'title' => __('版权声明开关', 'dmeng'),
      'description' => __('开关网站的版权声明（选择关闭将全部不显示，无论文章页怎么设置）。', 'dmeng')
    );

    $rows[] = array(
      'type' => 'select',
      'name' => 'post_index_all',
      'value' => $post_index_all,
      'options' => array(
        1 => __('全部都显示', 'dmeng'),
        2 => __('只在文章页显示', 'dmeng'),
        3 => __('只在页面显示', 'dmeng'),
        0 => __('不显示', 'dmeng')
      ),
      'title' => __('锚点导航开关', 'dmeng'),
      'description' => __('开关文章的锚点导航（选择关闭将全部不显示，无论文章页怎么设置）。', 'dmeng')
    );

    $rows[] = array(
      'type' => 'select',
      'name' => 'post_thumbnail[on]',
      'value' => (int)$post_thumbnail['on'],
      'options' => array(
        1 => __('只显示特色图像', 'dmeng'),
        2 => __('没有特色图像时显示文章的第一张图片', 'dmeng'),
        0 => __('不显示', 'dmeng')
      ),
      'title' => __('文章缩略图', 'dmeng'),
      'description' => __('在列表页显示文章缩略图（推荐设置220x146特色图像）。', 'dmeng')
    );

    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'post_thumbnail[suffix]',
      'value' => $post_thumbnail['suffix'],
      'title' => __('缩略图地址后缀', 'dmeng'),
      'description' => __('常用于缩略图处理。使用七牛云存储的童鞋直接使用默认值 ?imageView2/1/w/220/h/146/q/100 即可。', 'dmeng')
    );

    $rows[] = array(
      'type' => 'select',
      'name' => 'google_code_prettify',
      'value' => $google_code_prettify,
      'options' => array(
        1 => __('启用', 'dmeng'),
        0 => __('禁用', 'dmeng')
      ),
      'title' => __('高亮代码', 'dmeng'),
      'description' => __('在页面载入高亮代码。启用后将自动高亮&lt;pre&gt;标签中的内容', 'dmeng')
    );
    
    echo dmeng_options_form_output(
      array(
        'method' => 'post',
        'class' => 'options-box'
      ),
      dmeng_options_new_nonce(__FUNCTION__) . dmeng_options_table_build($rows)
    );

  }
  
  public function adsense_options() {

    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );

      foreach (array('adsense_archive', 'adsense_author', 'adsense_single') as $key) {
        if (isset($data[$key]))
          update_option('dmeng_'.$key, json_encode($data[$key]));
      }

      echo dmeng_options_notice();
    }

    $adsense_archive = (array)wp_unslash(json_decode(get_option('dmeng_adsense_archive','{"top":"","bottom":""}'), true));
    $adsense_author = (array)wp_unslash(json_decode(get_option('dmeng_adsense_author','{"top":"","bottom":""}'), true));
    $adsense_single = (array)wp_unslash(json_decode(get_option('dmeng_adsense_single', '{"top":"","comment":"","bottom":""}'), true));

    $rows = array();

    $rows[] = array(
      'type' => 'textarea',
      'name' => 'adsense_archive[top]',
      'value' => $adsense_archive['top'],
      'title' => __('归档页广告', 'dmeng'),
      'description' => __('分类/标签/搜索/日期归档页顶部', 'dmeng')
    );
    
    $rows[] = array(
      'type' => 'textarea',
      'name' => 'adsense_archive[bottom]',
      'value' => $adsense_archive['bottom'],
      'title' => '',
      'description' => __('分类/标签/搜索/日期归档页底部', 'dmeng')
    );

    $rows[] = array(
      'type' => 'textarea',
      'name' => 'adsense_author[top]',
      'value' => $adsense_author['top'],
      'title' => __('作者页广告', 'dmeng'),
      'description' => __('作者页顶部', 'dmeng')
    );
    
    $rows[] = array(
      'type' => 'textarea',
      'name' => 'adsense_author[bottom]',
      'value' => $adsense_author['bottom'],
      'title' => '',
      'description' => __('作者页底部', 'dmeng')
    );

    $rows[] = array(
      'type' => 'textarea',
      'name' => 'adsense_single[top]',
      'value' => $adsense_single['top'],
      'title' => __('内容页广告', 'dmeng'),
      'description' => __('文章/页面/附件页顶部', 'dmeng')
    );
    
    $rows[] = array(
      'type' => 'textarea',
      'name' => 'adsense_single[comment]',
      'value' => $adsense_single['comment'],
      'title' => '',
      'description' => __('文章/页面/附件页评论框上方', 'dmeng')
    );

    $rows[] = array(
      'type' => 'textarea',
      'name' => 'adsense_single[bottom]',
      'value' => $adsense_single['bottom'],
      'title' => '',
      'description' => __('文章/页面/附件页底部', 'dmeng')
    );
    
    echo dmeng_options_form_output(
      array(
        'method' => 'post',
        'class' => 'options-box'
      ),
      dmeng_options_new_nonce(__FUNCTION__) . dmeng_options_table_build($rows)
    );

  }

  public function discussion_options() {

    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );

      foreach (array('sticky_comment_title', 'sticky_comment_button_txt') as $key) {
        if (isset($data[$key]))
          update_option('dmeng_'.$key, sanitize_text_field($data[$key]));
      }
      
      update_option('dmeng_hide_comment_email', (int)$data['hide_comment_email']);
      
      echo dmeng_options_notice();
    }

    $sticky_comment_title = get_option('dmeng_sticky_comment_title', __('置顶评论','dmeng'));
    $sticky_comment_button_txt = get_option('dmeng_sticky_comment_button_txt',__('置顶','dmeng'));
    $hide_comment_email = (int)get_option('dmeng_hide_comment_email', 1);

    $rows = array();

    $rows[] = array(
      'type' => 'input',
      'name' => 'sticky_comment_title',
      'value' => $sticky_comment_title,
      'title' => __('置顶评论标题文本', 'dmeng'),
      'description' => ''
    );
    
    $rows[] = array(
      'type' => 'input',
      'name' => 'sticky_comment_button_txt',
      'value' => $sticky_comment_button_txt,
      'title' => __('置顶按钮文本', 'dmeng'),
      'description' => ''
    );
    
    $rows[] = array(
      'type' => 'select',
      'name' => 'hide_comment_email',
      'value' => $hide_comment_email,
      'options' => array(
        1 => __('隐藏', 'dmeng'),
        0 => __('不隐藏', 'dmeng')
      ),
      'title' => __('隐藏邮箱地址', 'dmeng'),
      'description' => __('自动隐藏评论中的邮箱地址，只有评论作者、文章作者或更高权限的用户可见。', 'dmeng')
    );

    echo dmeng_options_form_output(
      array(
        'method' => 'post',
        'class' => 'options-box'
      ),
      dmeng_options_new_nonce(__FUNCTION__) . dmeng_options_table_build($rows)
    );

  }
  
  public function open_options() {

    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );

      foreach (array('open_qq', 'open_weibo', 'open_sms', 'open_remember_the_user') as $key) {
        if (isset($data[$key]))
          update_option('dmeng_'.$key, (int)$data[$key]);
      }
      
      foreach (array('open_qq_id', 'open_qq_key', 'open_weibo_key', 'open_weibo_secret', 'open_sms_key', 'open_role') as $key) {
        if (isset($data[$key]))
          update_option('dmeng_'.$key, sanitize_text_field($data[$key]));
      }
      
      echo dmeng_options_notice();
    }

    $roles = array();
    $editable_roles = array_reverse( get_editable_roles() );
    foreach ( $editable_roles as $role => $details ) {
      $name = translate_user_role($details['name'] );
      $role = esc_attr($role);
      $roles[$role] = $name;
    }

    $rows = array();

    $rows[] = array(
      'type' => 'select',
      'name' => 'open_qq',
      'value' => (int)get_option('dmeng_open_qq', 1),
      'options' => array(
        1 => __('启用', 'dmeng'),
        0 => __('关闭', 'dmeng')
      ),
      'title' => __('启用QQ登录','dmeng').' [ <a href="http://www.dmeng.net/connect-qq.html" title="'.__('网站接入QQ登录申请','dmeng').'" target="_blank">?</a> ]',
      'description' => __('启用前提是设置了相应的 QQ ID 和 QQ KEY 。', 'dmeng')
    );
    
    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'open_qq_id',
      'value' => get_option('dmeng_open_qq_id'),
      'title' => __('QQ ID','dmeng'),
      'description' => ''
    );

    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'open_qq_key',
      'value' => get_option('dmeng_open_qq_key'),
      'title' => __('QQ KEY','dmeng'),
      'description' => ''
    );

    $rows[] = array(
      'type' => 'select',
      'name' => 'open_weibo',
      'value' => (int)get_option('dmeng_open_weibo', 1),
      'options' => array(
        1 => __('启用', 'dmeng'),
        0 => __('关闭', 'dmeng')
      ),
      'title' => __('启用微博登录','dmeng').' [ <a href="http://www.dmeng.net/connect-weibo.html" title="'.__('网站接入微博登录申请','dmeng').'" target="_blank">?</a> ]',
      'description' => __('启用前提是设置了相应的 WEIBO KEY 和 WEIBO SECRET 。', 'dmeng')
    );

    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'open_weibo_key',
      'value' => get_option('dmeng_open_weibo_key'),
      'title' => __('WEIBO KEY','dmeng'),
      'description' => ''
    );

    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'open_weibo_secret',
      'value' => get_option('dmeng_open_weibo_secret'),
      'title' => __('WEIBO SECRET','dmeng'),
      'description' => ''
    );
    
    $rows[] = array(
      'type' => 'select',
      'name' => 'open_sms',
      'value' => (int)get_option('dmeng_open_sms', 1),
      'options' => array(
        1 => __('启用', 'dmeng'),
        0 => __('关闭', 'dmeng')
      ),
      'title' => __('启用短信登录','dmeng'),
      'description' => __('默认使用聚合短信API发送短信。', 'dmeng')
    );

    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'open_sms_key',
      'value' => get_option('dmeng_open_sms_key'),
      'title' => __('短信 API KEY','dmeng'),
      'description' => ''
    );

    $rows[] = array(
      'type' => 'select',
      'name' => 'open_role',
      'value' => get_option('dmeng_open_role', 'contributor'),
      'options' => $roles,
      'title' => __('默认角色','dmeng'),
      'description' => __('新登录用户的角色，默认是投稿者。', 'dmeng')
    );

    $rows[] = array(
      'type' => 'select',
      'name' => 'open_remember_the_user',
      'value' => (int)get_option('dmeng_open_remember_the_user', 1),
      'options' => array(
        1 => __('启用', 'dmeng'),
        0 => __('关闭', 'dmeng')
      ),
      'title' => __('记住登录状态','dmeng'),
      'description' => __('使用开放平台登录是否记住登录状态，不记住的话当关闭浏览器窗口后需要重新登录。', 'dmeng')
    );
    
    echo dmeng_options_form_output(
      array(
        'method' => 'post',
        'class' => 'options-box'
      ),
      dmeng_options_new_nonce(__FUNCTION__) . dmeng_options_table_build($rows)
    );

  }  

  public function credit_options() {

    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );

      foreach (array('reg_credit', 'rec_post_credit', 'rec_post_num', 'rec_comment_credit', 'rec_comment_num', 'rec_reg_credit', 'rec_reg_num', 'rec_view_credit', 'rec_view_num') as $key) {
        if (isset($data[$key]))
          update_option('dmeng_'.$key, (int)$data[$key]);
      }
      
      echo dmeng_options_notice();
    }

    $rows = array();

    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'reg_credit',
      'value' => get_option('dmeng_reg_credit', '50'),
      'title' => __('新用户注册奖励','dmeng'),
      'description' => __('新用户注册自动奖励的积分','dmeng')
    );

    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'rec_post_credit',
      'value' => get_option('dmeng_rec_post_credit', '50'),
      'title' => __('投稿一次得分','dmeng'),
      'description' => ''
    );
    
    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'rec_post_num',
      'value' => get_option('dmeng_rec_post_num', '5'),
      'title' => __('每天投稿次数','dmeng'),
      'description' => __('每天可以获得积分奖励的投稿次数','dmeng')
    );  
    
    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'rec_comment_credit',
      'value' => get_option('dmeng_rec_comment_credit', '5'),
      'title' => __('评论一次得分','dmeng'),
      'description' => ''
    );   
    
    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'rec_comment_num',
      'value' => get_option('dmeng_rec_comment_num', '50'),
      'title' => __('每天评论次数','dmeng'),
      'description' => __('每天可以获得积分奖励的评论次数','dmeng')
    );
    
    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'rec_reg_credit',
      'value' => get_option('dmeng_rec_reg_credit', '50'),
      'title' => __('注册推广一次得分','dmeng'),
      'description' => ''
    );

    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'rec_reg_num',
      'value' => get_option('dmeng_rec_reg_num','5'),
      'title' => __('每天注册推广次数','dmeng'),
      'description' => __('每天可以获得积分奖励的注册推广次数','dmeng')
    );    
    
    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'rec_view_credit',
      'value' => get_option('dmeng_rec_view_credit', '5'),
      'title' => __('访问推广一次得分','dmeng'),
      'description' => ''
    );

    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'rec_view_num',
      'value' => get_option('dmeng_rec_view_num','50'),
      'title' => __('每天访问推广次数','dmeng'),
      'description' => __('每天可以获得积分奖励的访问推广次数','dmeng')
    );

    echo dmeng_options_form_output(
      array(
        'method' => 'post',
        'class' => 'options-box'
      ),
      dmeng_options_new_nonce(__FUNCTION__) . dmeng_options_table_build($rows)
    );

  }
  
  public function gift_options() {

    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );

      foreach (array('is_gift_open', 'is_gift_future', 'gift_num') as $key) {
        if (isset($data[$key]))
          update_option('dmeng_'.$key, (int)$data[$key]);
      }
      
      foreach (array('gift_filter', 'gift_tips', 'gift_notice') as $key) {
        if (isset($data[$key]))
          update_option('dmeng_'.$key, sanitize_text_field($data[$key]));
      }
      
      echo dmeng_options_notice();
    }

    $dmeng_is_gift_open = (int)get_option('dmeng_is_gift_open', 0);
    $dmeng_gift_filter = get_option('dmeng_gift_filter', '0-100,100-1000,1000-10000,10000-0');
    $dmeng_is_gift_future = (int)get_option('dmeng_is_gift_future', 1);
    $dmeng_gift_num = (int)get_option('dmeng_gift_num', 12);
    $dmeng_gift_tips = get_option('dmeng_gift_tips');
    $dmeng_gift_notice = get_option('dmeng_gift_notice');

    $rows = array();

    $rows[] = array(
      'type' => 'select',
      'name' => 'is_gift_open',
      'value' => $dmeng_is_gift_open,
      'options' => array(
        1 => __('启用', 'dmeng'),
        0 => __('关闭', 'dmeng')
      ),
      'title' => __('开启积分换礼','dmeng'),
      'description' => ($dmeng_is_gift_open ? sprintf(__('积分换礼地址是：%s', 'dmeng'), get_post_type_archive_link('gift')) : __('启用积分换礼功能。', 'dmeng'))
    );
    
    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'gift_filter',
      'value' => $dmeng_gift_filter,
      'title' => __('积分范围','dmeng'),
      'description' => __('上方筛选条件中的积分筛选，以英文 , 号分割，0-100代表100以下，100-1000代表100至1000，10000-0代表10000以上','dmeng')
    );

    $rows[] = array(
      'type' => 'select',
      'name' => 'is_gift_future',
      'value' => $dmeng_is_gift_future,
      'options' => array(
        1 => __('启用', 'dmeng'),
        0 => __('关闭', 'dmeng')
      ),
      'title' => __('定时发布','dmeng'),
      'description' => __('显示定时发布的礼品', 'dmeng')
    );

    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'gift_num',
      'value' => $dmeng_gift_num,
      'title' => __('显示礼品数量','dmeng'),
      'description' => __('列表页一页显示的礼品数量。因为一排4列，所以推荐设置可以被4整除的数字，默认是12。','dmeng')
    );

    $rows[] = array(
      'type' => 'textarea',
      'name' => 'gift_tips',
      'value' => $dmeng_gift_tips,
      'title' => __('温馨提示','dmeng'),
      'description' => __('礼品详细信息下的提示语。','dmeng')
    );

    $rows[] = array(
      'type' => 'textarea',
      'name' => 'gift_notice',
      'value' => $dmeng_gift_notice,
      'title' => __('兑换须知','dmeng'),
      'description' => __('礼品兑换须知','dmeng')
    );

    echo dmeng_options_form_output(
      array(
        'method' => 'post',
        'class' => 'options-box'
      ),
      dmeng_options_new_nonce(__FUNCTION__) . dmeng_options_table_build($rows)
    );

  }
  
  public function smtp_options() {

    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );

      update_option('dmeng_smtp', json_encode(array_map('sanitize_text_field', $data['dmeng_smtp'])));

      echo dmeng_options_notice();
    }

    $smtp = wp_parse_args(
      (array)wp_unslash(json_decode(get_option('dmeng_smtp'), true)),
      array(
        'option' => 0,
        'host' => '',
        'ssl' => 0,
        'port' => 25,
        'user' => '',
        'pass' => '',
        'name' => ''
      )
    );

    $rows = array();

    $rows[] = array(
      'type' => 'select',
      'name' => 'dmeng_smtp[option]',
      'value' => (int)$smtp['option'],
      'options' => array(
        1 => __('启用', 'dmeng'),
        0 => __('关闭', 'dmeng')
      ),
      'title' => __('启用SMTP','dmeng'),
      'description' => ''
    );
    
    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'dmeng_smtp[host]',
      'value' => $smtp['host'],
      'title' => __('发信服务器','dmeng'),
      'description' => ''
    );  
    
    $rows[] = array(
      'type' => 'select',
      'name' => 'dmeng_smtp[ssl]',
      'value' => (int)$smtp['ssl'],
      'options' => array(
        1 => __('启用', 'dmeng'),
        0 => __('关闭', 'dmeng')
      ),
      'title' => __('启用SSL','dmeng'),
      'description' => ''
    );
    
    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'dmeng_smtp[port]',
      'value' => $smtp['port'],
      'title' => __('端口号','dmeng'),
      'description' => ''
    );

    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'dmeng_smtp[user]',
      'value' => $smtp['user'],
      'title' => __('发信账号','dmeng'),
      'description' => ''
    );
    
    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => 'dmeng_smtp[pass]',
      'value' => $smtp['pass'],
      'title' => __('账号密码','dmeng'),
      'description' => ''
    );   
    
    $rows[] = array(
      'type' => 'input',
      'name' => 'dmeng_smtp[name]',
      'value' => $smtp['name'],
      'title' => __('显示名称','dmeng'),
      'description' => ''
    );

    echo dmeng_options_form_output(
      array(
        'method' => 'post',
        'class' => 'options-box'
      ),
      dmeng_options_new_nonce(__FUNCTION__) . dmeng_options_table_build($rows)
    );

  }
  
  public function seo_options() {

    global $dmeng_Sitemap;

    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );

      if (isset($data[$dmeng_Sitemap->baidu_api_field_name]))
        update_option($dmeng_Sitemap->baidu_api_field_name, esc_url($data[$dmeng_Sitemap->baidu_api_field_name]));

      update_option($dmeng_Sitemap->setting_field_name, 
        (isset($data[$dmeng_Sitemap->setting_field_name]) ? 1 : 0)
      );    

      if (isset($data['seo_meta']))
        update_option('dmeng_seo_meta', (int)$data['seo_meta']);

      echo dmeng_options_notice();
    }

    $rows = array();

    $rows[] = array(
      'value' => ( (int)get_option('blog_public') ? __('已打开','dmeng') : __('请到“设置 > 阅读 > 对搜索引擎的可见性”打开','dmeng') ),
      'title' => __('对搜索引擎的可见性','dmeng'),
      'description' => ''
    );
    
    $rows[] = array(
      'value' => '<a href="'.$dmeng_Sitemap->get_sitemap_index_url().'" target="_blank">'.$dmeng_Sitemap->get_sitemap_index_url().'</a>',
      'title' => __('地图索引地址','dmeng'),
      'description' => ''
    );

    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => $dmeng_Sitemap->baidu_api_field_name,
      'value' => get_option($dmeng_Sitemap->baidu_api_field_name),
      'title' => __('百度推送接口','dmeng'),
      'description' => '<label for="'.$dmeng_Sitemap->setting_field_name.'">
        <input name="'.$dmeng_Sitemap->setting_field_name.'" type="checkbox" id="'.$dmeng_Sitemap->setting_field_name.'" '.((int)get_option($dmeng_Sitemap->setting_field_name) ? 'checked="checked"' : '').'> '.__('发布文章时自动通知百度', 'dmeng').' [ <a href="http://zhanzhang.baidu.com/college/courseinfo?id=267&page=2#h2_article_title9" target="_blank">?</a> ]
      </label>'
    );  
    
    $rows[] = array(
      'value' => '<button type="button" class="button button-primary ping-baidu">'.__('推送今天更新的文章', 'dmeng').'</button> <button type="button" class="button button-primary ping-baidu" data-index="sitemap">'.__('推送地图索引', 'dmeng').'</button>',
      'title' => __('推送到百度','dmeng'),
      'description' => ''
    );

    $rows[] = array(
      'type' => 'select',
      'name' => 'seo_meta',
      'value' => (int)get_option('dmeng_seo_meta', 1),
      'options' => array(
        1 => __('启用', 'dmeng'),
        0 => __('关闭', 'dmeng')
      ),
      'title' => __('使用关键词描述','dmeng'),
      'description' => __('如果使用了插件管理网页关键词和描述，可将主题自带的关闭','dmeng')
    );

    ob_start();
    ?>
        <script>
        var pinging = false;
        !function($){
          $('.ping-baidu').on('click', function(){
            if (pinging) {
              return;
            }
            pinging = true;
            var $btn = $(this);
            $btn.html('<?php _e('推送中…请稍后…', 'dmeng');?>').addClass('disabled').prop('disabled', true);
            $.ajax({
              type: 'POST',
              url: ajaxurl,
              data: { 'action':'<?php echo $dmeng_Sitemap->setting_field_name;?>', 'nonce':'<?php echo wp_create_nonce($dmeng_Sitemap->setting_field_name);?>', 'index':$(this).data('index') },
              complete: function (e) {
                $btn.html(e.responseText ? e.responseText : '<?php _e('出错了，请刷新重试', 'dmeng');?>');
                pinging = false;
              }
            });
            return false;
          });
        }(jQuery);
        </script>
     <?php
    $script = ob_get_clean();

    echo dmeng_options_form_output(
      array(
        'method' => 'post',
        'class' => 'options-box'
      ),
      dmeng_options_new_nonce(__FUNCTION__) . dmeng_options_table_build($rows) . $script
    );

  }
  
  public function mobile_options() {

    $theme_field_name = 'dmeng_mobile_theme';
    $home_field_name = 'dmeng_mobile_home';
    $siteurl_field_name = 'dmeng_mobile_siteurl';
  
    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );

      if (isset($data[$theme_field_name])) {
        $new_mobile_theme = $data[$theme_field_name];
        if (isset($themes[$new_mobile_theme]))
          update_option( $theme_field_name, $new_mobile_theme );
      }

      if (isset($data[$home_field_name])) {
        update_option($home_field_name, esc_url($data[$home_field_name]));
      }

      if (isset($data[$siteurl_field_name])) {
        update_option($siteurl_field_name, esc_url($data[$siteurl_field_name]));
      }
      
      echo dmeng_options_notice();
    }

    $themes = wp_get_themes();
    $options = array();
    foreach ($themes as $name=>$theme) {
      if ('dmeng2.1'!=$theme->get('Template'))
        continue;
      $options[$name] = $theme->get('Name').'('.$name.')';
    }
    $have_themes = !empty($options);
    
    $mobile_theme = get_option($theme_field_name, 'dmeng2.1-mobile');
    $mobile_home = get_option($home_field_name);
    $mobile_siteurl = get_option($siteurl_field_name);
    
    $rows = array();

    $rows[] = array(
      'value' => (true===apply_filters('using_dmeng_theme_switch', '__return_false') ? __('已启用','dmeng') : __('请安装并启用！','dmeng')),
      'title' => __('切换插件','dmeng'),
      'description' => __('根据WordPress的工作机制，要完整地完成主题切换，需要使用插件实现。','dmeng')
    );

    $rows[] = array(
      'type' => ($have_themes ? 'select' : ''),
      'name' => $theme_field_name,
      'value' => ($have_themes ? $mobile_theme : __('请上传多梦主题2.1的子主题', 'dmeng')),
      'options' => $options,
      'title' => __('选择主题','dmeng'),
      'description' => ''
    );
    
    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => $siteurl_field_name,
      'value' => $mobile_siteurl,
      'title' => __('WordPress地址（URL）','dmeng'),
      'description' => __('移动端的WordPress地址（URL），如果没有绑定独立域名可留空','dmeng')
    ); 
    
    $rows[] = array(
      'type' => 'input',
      'classes' => 'code',
      'name' => $home_field_name,
      'value' => $mobile_home,
      'title' => __('站点地址（URL）','dmeng'),
      'description' => __('移动端的站点地址（URL），如果没有绑定独立域名可留空','dmeng')
    );  

    echo dmeng_options_form_output(
      array(
        'method' => 'post',
        'class' => 'options-box'
      ),
      dmeng_options_new_nonce(__FUNCTION__) . dmeng_options_table_build($rows)
    );
  }

}

new dmeng_Options_Default;