<?php

class dmeng_Options_Shortcode {
  
  public $tabs;

  public function __construct() {
    
    $this->tabs = array(
      'dmengslide' => __('幻灯片', 'dmeng'),
      'dmengl2v' => __('登录可见', 'dmeng'),
      'dmengr2v' => __('登录评论可见', 'dmeng'),
      'dmenglist' => __('文章列表', 'dmeng'),
      'dmengterms' => __('分类列表', 'dmeng'),
      'dmengrank' => __('排行榜', 'dmeng'),
      'dmengfile' => __('附件', 'dmeng'),
      'dmengaction' => __('同步动作', 'dmeng')
    );

    add_filter('dmeng_options_shortcode_page-tabs', array($this, 'add_tab'));

    foreach (array_keys($this->tabs) as $key) {
      add_action('dmeng_options_shortcode_page_'.$key, array($this, $key.'_options'));
    }

  }

  public function add_tab($tabs) {
    $tabs = array_merge($this->tabs, $tabs);
    return $tabs;
  }
  
  public function dmengslide_options() {

    $action = 'list';

    $msg = '';
    $success = true;
    
    if (!empty($_POST['data'])) {

      dmeng_options_check_nonce(__FUNCTION__);

      parse_str( $_POST['data'], $data );

      if (
        !empty($data['new'])
        && !empty($data['_wpnonce'])
        && is_numeric($data['new'])
        && wp_verify_nonce($data['_wpnonce'], 'new')
      ) {
        
        $action = 'new';
        // 新建
        
      } else if (
        !empty($data['edit'])
        && !empty($data['_wpnonce'])
        && is_numeric($data['edit'])
        && wp_verify_nonce($data['_wpnonce'], 'edit_'.$data['edit'])
      ) {
        
        $action = 'edit';
        // 编辑
        
      } else if (
        !empty($data['update'])
        && !empty($data['_wpnonce'])
        && is_numeric($data['update'])
        && wp_verify_nonce($data['_wpnonce'], 'update_'.$data['update'])
      ) {
        
        // 编辑数据
        $action = 'update';
        
        if (
          !empty($data['img'])
          && !empty($data['url'])
          && !empty($data['title'])
          && !empty($data['desc'])
        ) {

          $slide_id = $data['update'];

          $image = $url = $title = $desc = array();

          foreach ($data['img'] as $key=>$img) {
            if (empty($img))
              continue;
            $image[] = $img;
            $url[] = $data['url'][$key];
            $title[] = $data['title'][$key];
            $desc[] = $data['desc'][$key];
          }
          
          update_option('dmeng_slide_'.$slide_id, json_encode(array(
            'name' => sanitize_text_field($data['name']),
            'img' => $image,
            'url' => $url,
            'title' => $title,
            'desc' => $desc
          )));
          
          $slides = (array)wp_unslash(json_decode(get_option('dmeng_slides'), true));
          if (false===in_array($slide_id, $slides)) {
            $slides[] = $slide_id;
            update_option('dmeng_slides', json_encode($slides));
          }

          $msg = __('幻灯片更新成功', 'dmeng');

        }

      } else if (
        !empty($data['delete'])
        && !empty($data['_wpnonce'])
        && is_numeric($data['delete'])
        && wp_verify_nonce($data['_wpnonce'], 'delete_'.$data['delete'])
      ) {
        
        // 删除
        $action = 'delete';
        delete_option( 'dmeng_slide_'.$data['delete'] );
        $msg = sprintf(__('操作成功，ID %s 幻灯片已被删除', 'dmeng'), '<b>'.$data['delete'].'</b>' );
      }
      
      if ($msg)
        echo dmeng_options_notice($msg, $success);

    }

    // 提示
    echo '<p class="options-box">'.sprintf(__( '小提示：为了保证最佳的用户体验，请以 750px 作为图片宽度（主内容区域宽度），同一组的图片采用一致高度，图片编辑完成后使用压缩工具压缩以提升加载速度，如 %1$s 可减少图片 70%% 体积。如果你不能保证图片的尺寸和质量，那么我们不建议你使用幻灯片，糟糕的幻灯片会严重影响网站的美观性！', 'dmeng' ), '<a href="https://tinypng.com/" target="_blank">TinyPNG</a>').'</p>';
    
    // 页面 nonce
    echo dmeng_options_new_nonce(__FUNCTION__);

    if (
      'new'==$action
      || 'edit'==$action
      || 'update'==$action
    ) {

      if ('new'==$action) {
        $slide_id = current_time('timestamp');
        $slide = array();
      } else {
        $slide_id = (int)$data[$action];
        $slide = (array)wp_unslash(json_decode(get_option('dmeng_slide_'.$slide_id), true));
      }

      $slide = array_merge(
        array(
          'name' => '',
          'img' => array(),
          'url' => array(),
          'title' => array(),
          'desc' => array()
        ), $slide
      );

      echo 
        dmeng_options_form_output(
          array( 'method' => 'post', 'style' => 'display:inline' ),
          '<input class="button" type="submit" value="'.__('返回列表', 'dmeng').'" />',
          false
        ) . 
        dmeng_options_form_output(
          array( 'method' => 'post', 'style' => 'display:inline' ),
          '<input type="hidden" name="edit" value="'.$slide_id.'" />
          <input type="hidden" name="_wpnonce" value="'.wp_create_nonce('edit_'.$slide_id).'" />
          <input class="button" type="submit" value="'.__('刷新', 'dmeng').'" />',
          false
        ) . 
        dmeng_options_form_output(
          array( 'method' => 'post', 'style' => 'display:inline' ),
          '<input type="hidden" name="delete" value="'.$slide_id.'" />
          <input type="hidden" name="_wpnonce" value="'.wp_create_nonce('delete_'.$slide_id).'" />
          <input class="button confirm_action" type="submit" value="'.__('删除', 'dmeng').'" data-confirm-title="'.__('删除后不可恢复，你确定删除吗？', 'dmeng').'" />',
          false
        );

        echo '<div class="options-box hide" id="example-slide">'.dmeng_options_table_build(array(
          array(
            'value' => '<input name="img[]" type="text" value="" class="regular-text ltr" required="required"> 
            <a href="javascript:;" class="button slide_upload_button">'.__('选择或上传', 'dmeng').'</a> 
            <a href="javascript:;" class="button slide_preview_button">'.__('预览', 'dmeng').'</a>',
            'title' => __('图片（必须）', 'dmeng')
          ), 
          array(
            'type' => 'input',
            'name' => 'url[]',
            'value' => '',
            'title' => __('链接', 'dmeng')
          ),
          array(
            'type' => 'input',
            'name' => 'title[]',
            'value' => '',
            'title' => __('标题', 'dmeng')
          ),
          array(
            'type' => 'input',
            'name' => 'desc[]',
            'value' => '',
            'title' => __('描述', 'dmeng')
          ),
          array(
            'value' => '
              <a href="javascript:;" class="button slide_up">'.__('往上移', 'dmeng').'</a>
              <a href="javascript:;" class="button slide_down">'.__('往下移', 'dmeng').'</a>
              <a href="javascript:;" class="button slide_delete">'.__('删除', 'dmeng').'</a>
              <a href="javascript:;" class="button slide_add">'.__('添加一项', 'dmeng').'</a>
					  ',
            'title' => __('操作', 'dmeng')
          ),
        )).'</div>'.
        dmeng_options_form_output(
          array( 'method' => 'post', 'id' => 'slideform', 'style' => 'margin:0;padding:0' ),
          '<h3>'.__('正在编辑幻灯片组', 'dmeng').'</h3>'.
          '<div class="options-box">'.
          dmeng_options_table_build(array(
            array(
              'type' => 'input',
              'name' => 'name',
              'value' => $slide['name'],
              'title' => __('名称', 'dmeng'),
              'description' => __('仅作为列表页显示区分', 'dmeng')
            ),
            array(
              'value' => '<input value="'.esc_attr('[dmengslide id="'.$slide_id.'"]').'" type="text" class="regular-text ltr" disabled="disabled" />',
              'title' => __('短代码', 'dmeng'),
              'description' => __('在文章内容等地方调用使用的短代码', 'dmeng')
            )
          )).
          '</div><div id="afterslide"></div>
          <input type="hidden" name="update" value="'.$slide_id.'" />
          <input type="hidden" name="_wpnonce" value="'.wp_create_nonce('update_'.$slide_id).'" />
          <input class="button button-primary" type="submit" value="'.__('保存更改', 'dmeng').'" /> 
          <a href="javascript:;" class="button slide_new">'.__('添加一项', 'dmeng').'</a>
          ',
          false
        );
          
        if ($slide['img']) {
          foreach ($slide['img'] as $key=>$img) {
            if (empty($img))
              continue;
            
            echo '
            <div class="slide-item hide">
              <b class="img">'.$img.'</b>
              <b class="url">'.$slide['url'][$key].'</b>
              <b class="title">'.$slide['title'][$key].'</b>
              <b class="desc">'.$slide['desc'][$key].'</b>
            </div>';
          }
        }
          
?>
<script type="text/javascript">
!function($){
$(document).ready(function($){

  window.newSlide = function(img, url, title, desc) {
    var $slide = $('#example-slide').clone(true),
        data = {'img':img, 'url':url, 'title':title, 'desc':desc};
    $slide.attr('id', null).removeClass('hide').addClass('slide-box');
    for (key in data) {
      if (data[key]) {
        $slide.find('[name*='+key+']').val(data[key]);
      }
    }
    return $slide;
  };

  $.each($('.slide-item'), function(){
    $('#afterslide').before(newSlide(
      $(this).find('.img').html(),
      $(this).find('.url').html(),
      $(this).find('.title').html(),
      $(this).find('.desc').html()
    ));
    $(this).remove();
  });

	var upload_frame;   
  $('.slide_upload_button').on('click', function(event) {
    event.preventDefault();
    var $input = $(this).siblings('input');
    upload_frame = wp.media({
      multiple: false,
      library : {
        type : 'image'
      }
    });   
    upload_frame.on('select',function(){
      attachment = upload_frame.state().get('selection').first().toJSON();
      $input.val(attachment.url);
    });   
    upload_frame.open();
  });
  $('.slide_preview_button').on('click', function() {
		var $input = $(this).siblings('input');
    if ($input.val()) {
      window.open($input.val());
    } else {
      $input.focus();
    }
  });
  $('.slide_up').on('click', function() {
		var $table = $(this).parents('.slide-box');
		$table.prev('.slide-box').before($table);
  });
  $('.slide_down').on('click', function() {
		var $table = $(this).parents('.slide-box');
		$table.next('.slide-box').after($table);
  });
  $('.slide_delete').on('click', function() {
    if ($('.slide-box').length>1) {
      $(this).parents('.slide-box').remove();;
    } else {
      alert('<?php _e('一组幻灯片至少要有一个幻灯片', 'dmeng');?>');
    }
  });
  $('.slide_add').on('click', function() {
    $(this).parents('.slide-box').after(newSlide());
  });
  $('.slide_new').on('click', function() {
    $('#afterslide').before(newSlide());
  });
});
}(jQuery);
</script>
<?php

    } else {

      echo dmeng_options_form_output(
            array('method' => 'post'),
            '<input type="hidden" name="new" value="1" />
            <input type="hidden" name="_wpnonce" value="'.wp_create_nonce('new').'" />
            <input class="button button-primary" type="submit" value="'.__('新建一组幻灯片', 'dmeng').'" />',
            false
           );
    
      $slides = (array)wp_unslash(json_decode(get_option('dmeng_slides'), true));
      if ($slides) {
        $filter_slides = array_flip($slides);
        foreach ($slides as $slide_id) {
          $slide = (array)wp_unslash(json_decode(get_option('dmeng_slide_'.$slide_id), true));
          if (
            empty($slide)
            || empty($slide['img'])
            || false===is_array($slide['img'])
          ) {
            unset($filter_slides[$slide_id]);
            continue;
          }

          $row = array();

          $edit_form = dmeng_options_form_output(
            array( 'method' => 'post', 'style' => 'display:inline' ),
            '<input type="hidden" name="edit" value="'.$slide_id.'" />
            <input type="hidden" name="_wpnonce" value="'.wp_create_nonce('edit_'.$slide_id).'" />
            <input class="button" type="submit" value="'.__('编辑', 'dmeng').'" />',
            false
          );
          
          $delete_form = dmeng_options_form_output(
            array( 'method' => 'post', 'style' => 'display:inline' ),
            '<input type="hidden" name="delete" value="'.$slide_id.'" />
            <input type="hidden" name="_wpnonce" value="'.wp_create_nonce('delete_'.$slide_id).'" />
            <input class="button confirm_action" type="submit" value="'.__('删除', 'dmeng').'" data-confirm-title="'.__('删除后不可恢复，你确定删除吗？', 'dmeng').'" />',
            false
          );
          
          $row[] = array(
            'value' => '<input value="'.esc_attr('[dmengslide id="'.$slide_id.'"]').'" type="text" class="regular-text ltr" disabled="disabled" />'.$edit_form.$delete_form,
            'title' => $slide['name']
          );

          $image = '';
          foreach ($slide['img'] as $img) {
            $image .= '<img src="'.$img.'" width="200" style="margin-right:10px;border:0" />';
          }
          $row[] = array(
            'value' => $image,
            'title' => __('预览', 'dmeng')
          );

          echo '<div class="options-box">'.dmeng_options_table_build($row).'</div>';
          
        }
        
        $filter_slides = array_flip($filter_slides);
        if ($filter_slides!=$slides) {
          update_option('dmeng_slides', json_encode($filter_slides));
        }

      }
    
    }
  }
  
  public function dmengl2v_options() {
    echo '<div class="options-box">'.dmeng_options_table_build(array(
      array(
        'title' => __('说明', 'dmeng'),
        'value' => sprintf(__('用户登录后可见，未登录用户会提示 %s 并链接到登录页面', 'dmeng'), '<span style="color:#a94442;background:#f2dede;padding:0 .25em;margin:0 .25em">' . __('[此处内容登录可见]','dmeng') . '</span>')
      ),
      array(
        'title' => __('用法', 'dmeng'),
        'value' => __('使用<b>[dmengl2v]</b>和<b>[/dmengl2v]</b>包含需要隐藏的内容，且前后无空格', 'dmeng')
      ),
      array(
        'title' => __('示例', 'dmeng'),
        'value' => __('[dmengl2v]这里的内容需要登录后才可见。[/dmengl2v]', 'dmeng')
      )
    )).'</div>';
  }
  
  public function dmengr2v_options() {
    echo '<div class="options-box">'.dmeng_options_table_build(array(
      array(
        'title' => __('说明', 'dmeng'),
        'value' => sprintf(__('用户登录后可见，未登录用户会提示 %s 并链接到登录页面', 'dmeng'), '<span style="color:#a94442;background:#f2dede;padding:0 .25em;margin:0 .25em">' . __('[此处内容登录并发表评论可见]','dmeng') . '</span>')
      ),
      array(
        'title' => __('注意', 'dmeng'),
        'value' => __('仅在文章/页面内容中有效，其他地方使用正常返回内容。', 'dmeng')
      ),
      array(
        'title' => __('用法', 'dmeng'),
        'value' => __('使用<b>[dmengr2v]</b>和<b>[/dmengr2v]</b>包含需要隐藏的内容，且前后无空格', 'dmeng')
      ),
      array(
        'title' => __('示例', 'dmeng'),
        'value' => __('[dmengr2v]这里的内容需要登录并发布评论后才可见。[/dmengr2v]', 'dmeng')
      )
    )).'</div>';
  }
  
  public function dmenglist_options() {
    echo '<div class="options-box">'.dmeng_options_table_build(array(
      array(
        'title' => __('说明', 'dmeng'),
        'value' => __('通过短代码调用文章列表', 'dmeng')
      ),
      array(
        'title' => __('参数', 'dmeng'),
        'value' => 
          '<b>'.__('style', 'dmeng').'</b> => '.__('列表样式，可选 list / thumbnail / archive', 'dmeng').'<br />'.
          '<b>'.__('sticky_posts', 'dmeng').'</b> => '.__('只显示置顶文章，可选 0 / 1', 'dmeng').'<br />'.
          '<b>'.__('taxonomy', 'dmeng').'</b> => '.__('分类法别名', 'dmeng').'<br />'.
          '<b>'.__('terms', 'dmeng').'</b> => '.__('分类ID，多个以 , 隔开', 'dmeng').'<br />'.
          '<b>'.__('post_type', 'dmeng').'</b> => '.__('文章类型', 'dmeng').'<br />'.
          '<b>'.__('number', 'dmeng').'</b> => '.__('显示数量', 'dmeng').'<br />'
 
      ),
      array(
        'title' => __('示例', 'dmeng'),
        'value' => __('[dmenglist style="list" terms="84"]', 'dmeng')
      ),
      array(
        'title' => __('备注', 'dmeng'),
        'value' => __('测试版暂不提供可视化生成工具，且正式版参数可能会有所调整', 'dmeng')
      )
    )).'</div>';
  }
  
  public function dmengterms_options() {
    echo '<div class="options-box">'.dmeng_options_table_build(array(
      array(
        'title' => __('说明', 'dmeng'),
        'value' => __('通过短代码调用分类列表', 'dmeng')
      ),
      array(
        'title' => __('参数', 'dmeng'),
        'value' => 
          '<b>'.__('style', 'dmeng').'</b> => '.__('列表样式，可选 cloud', 'dmeng').'<br />'.
          '<b>'.__('taxonomy', 'dmeng').'</b> => '.__('分类法别名', 'dmeng').'<br />'.
          '<b>'.__('include', 'dmeng').'</b> => '.__('必须包含分类ID，多个以 , 隔开', 'dmeng').'<br />'.
          '<b>'.__('exclude', 'dmeng').'</b> => '.__('排除分类ID，多个以 , 隔开', 'dmeng').'<br />'.
          '<b>'.__('number', 'dmeng').'</b> => '.__('显示数量', 'dmeng').'<br />'
 
      ),
      array(
        'title' => __('示例', 'dmeng'),
        'value' => __('[dmengterms number="15"]', 'dmeng')
      ),
      array(
        'title' => __('备注', 'dmeng'),
        'value' => __('测试版暂不提供可视化生成工具，且正式版参数可能会有所调整', 'dmeng')
      )
    )).'</div>';
  }
  
  public function dmengrank_options() {
    echo '<div class="options-box">'.dmeng_options_table_build(array(
      array(
        'title' => __('说明', 'dmeng'),
        'value' => __('通过短代码调用排行榜', 'dmeng')
      ),
      array(
        'title' => __('参数', 'dmeng'),
        'value' => 
          '<b>'.__('type', 'dmeng').'</b> => '.__('类型，可选 search / posts ', 'dmeng').'<br />'.
          '<b>'.__('number', 'dmeng').'</b> => '.__('显示数量', 'dmeng').'<br />'
 
      ),
      array(
        'title' => __('示例', 'dmeng'),
        'value' => __('[dmengrank number="6"]', 'dmeng')
      ),
      array(
        'title' => __('备注', 'dmeng'),
        'value' => __('测试版暂不提供可视化生成工具，且正式版参数可能会有所调整', 'dmeng')
      )
    )).'</div>';
  }
  
  public function dmengfile_options() {
    echo '<div class="options-box">'.dmeng_options_table_build(array(
      array(
        'title' => __('说明', 'dmeng'),
        'value' => __('通过短代码调用附件信息', 'dmeng')
      ),
      array(
        'title' => __('参数', 'dmeng'),
        'value' => 
          '<b>'.__('id', 'dmeng').'</b> => '.__('附件ID', 'dmeng')
      ),
      array(
        'title' => __('示例', 'dmeng'),
        'value' => __('[dmengfile id="16"]', 'dmeng')
      ),
      array(
        'title' => __('备注', 'dmeng'),
        'value' => __('测试版暂不提供可视化生成工具，且正式版参数可能会有所调整', 'dmeng')
      )
    )).'</div>';
  }
  
  public function dmengaction_options() {
    echo '<div class="options-box">'.dmeng_options_table_build(array(
      array(
        'title' => __('说明', 'dmeng'),
        'value' => __('在用户发表评论或兑换礼品等操作的同时做其他动作，测试版仅支持积分操作', 'dmeng')
      ),
      array(
        'title' => __('参数', 'dmeng'),
        'value' => 
          '<b>'.__('time', 'dmeng').'</b> => '.__('用户第几次操作时运行，留空或设置为0时代表每一次', 'dmeng').'<br />'.
          '<b>'.__('do', 'dmeng').'</b> => '.__('动作名称，可选 credit ', 'dmeng').'<br />'.
          '<b>'.__('plus', 'dmeng').'</b> => '.__('添加的积分', 'dmeng').'<br />'.
          '<b>'.__('minus', 'dmeng').'</b> => '.__('消费的积分', 'dmeng').'<br />'.
          '<b>'.__('title', 'dmeng').'</b> => '.__('备注', 'dmeng').'<br />'
      ),
      array(
        'title' => __('示例', 'dmeng'),
        'value' => __('由于功能较为复杂，正式版将提供可视化工具生成，测试版建议暂不使用', 'dmeng')
      ),
      array(
        'title' => __('备注', 'dmeng'),
        'value' => __('测试版暂不提供可视化生成工具，且正式版参数可能会有所调整', 'dmeng')
      )
    )).'</div>';
  }

}

new dmeng_Options_Shortcode;