<?php

/**
 * 
 */

class dmeng_Message {
  
  public $db_version = 150921;
  
  public function __construct() {

    add_action( 'dmeng_db_setup', array( $this, 'db_setup' ) );
    
    if ( is_admin() && (int)get_option( $this->table_name.'_db_version' ) < $this->db_version ) {
      add_filter( 'dmeng_db_upgrade_check', '__return_true' );
      add_action( 'dmeng_db_upgrade', array( $this, 'db_upgrade' ) );
    }
    
    add_action( 'delete_user', array( $this, 'delete_by_user' ) );
    
    add_action( 'wp_ajax_dmeng_get_message', array( $this, 'get_message_ajax_action' ) );
    add_action( 'wp_ajax_dmeng_pm', array( $this, 'pm_ajax_action' ) );

  }
  
  /**
   * 数据库表格名称
   */
  public $table_name = 'dmeng_message';   

  /**
   * 创建数据库表格
   * msg_id 自动增长主键
   * user_id 用户ID
   * msg_type 类型
   * msg_date 日期
   * msg_title 标题
   * msg_content 内容
   * 
   */
  public function db_setup() {
    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;
    if ( $wpdb->get_var("SHOW TABLES LIKE '$table_name'")==$table_name )
      return false;

    $charset_collate = '';
    if ( ! empty($wpdb->charset) )
      $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
    if ( ! empty($wpdb->collate) )
      $charset_collate .= " COLLATE $wpdb->collate";

    $result = $wpdb->query(
      "
        CREATE TABLE `$table_name` (
          `msg_id` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
          `user_id` bigint(20) unsigned DEFAULT NULL,
          `msg_type` varchar(20) DEFAULT NULL,
          `msg_date` datetime DEFAULT NULL,
          `msg_title` tinytext,
          `msg_content` text,
          PRIMARY KEY (`msg_id`),
          KEY `user_id` (`user_id`),
          KEY `msg_type` (`msg_type`)
        ) $charset_collate;
      "
    );
    
    if (!$result)
      return false;

    update_option( $this->table_name.'_db_version', $this->db_version );
    return true;
  }
  
  public function db_upgrade() {
    
    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;
    
    if (true===$this->db_setup()) {
      echo '<p style="color:#090">'.sprintf(__('已成功创建 %s 数据表。', 'dmeng'), $table_name).'</p>';
      return;
    }

    $num = (int)$wpdb->get_var("SELECT count(*) FROM $table_name ");

    $index = array();
    $result = $wpdb->get_results("SHOW INDEX FROM $table_name");
    if ($result) {
      foreach ($result as $item) {
        if ('PRIMARY'!=$item->Key_name)
          $index[] = $item->Key_name;
      }
    }
    
    $upgrade_sql =
      "ALTER TABLE `$table_name` 
        ".(in_array('user_id', $index) ? 'DROP INDEX `user_id`,' : '')."
        ".(in_array('msg_type', $index) ? 'DROP INDEX `msg_type`,' : '')."
        MODIFY COLUMN `msg_id` bigint(20) UNSIGNED NOT NULL AUTO_INCREMENT FIRST,
        MODIFY COLUMN `user_id` bigint(20) UNSIGNED NULL AFTER `msg_id`,
        ADD INDEX `user_id`(`user_id`),
        ADD INDEX `msg_type`(`msg_type`);
      ";

    if ($num>100000) {

      echo '<p style="color:#d54e21">'.sprintf(__('由于 %s 数据表过大，没有执行在线更新，建议手动更新。', 'dmeng'), $table_name).'</p>';
      
      echo '<p>'.__('MySQL更新代码可参考：', 'dmeng').'</p><pre><code>'.$upgrade_sql.'</code></pre>';
      
      return;
      
    } else {

      $result = $wpdb->query($upgrade_sql);
      
      if (!$result) {
        echo '<p style="color:#d54e21">'.sprintf(__('更新 %s 数据表失败，请重试。', 'dmeng'), $table_name).'</p>';
        return;
      }
      
      // 别忘了更新数据表版本号
      update_option( $this->table_name.'_db_version', $this->db_version );
      
      echo '<p style="color:#090">'.sprintf(__('已成功更新 %s 数据表。', 'dmeng'), $table_name).'</p>';
    }

  }

  public function add( $uid=0, $type='', $date='', $title='', $content='' ) {

    $uid = intval($uid);
    $title = sanitize_text_field($title);
    
    if ( !$uid || empty($title) )
      return;

    $type = $type ? sanitize_text_field($type) : 'unread';
    $date = $date ? $date : current_time('mysql');
    $content = htmlspecialchars($content);
    
    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;

    if ( $wpdb->insert( $table_name, array( 
      'user_id'=> $uid, 
      'msg_type'=> $type, 
      'msg_date'=> $date, 
      'msg_title'=> $title, 
      'msg_content'=> $content 
    ) ) ) {
      return $wpdb->insert_id;
    }
    
    return 0;
  }

  public function delete( $id=0, $uid=0, $type='', $title='' ) {

    $id = absint($id);
    $uid = absint($uid);

    if ( !$id && !$uid )
      return;

    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;

    $where = array();

    if ($id)
      $where['msg_id'] = $id;

    if ($uid)
      $where['user_id'] = $uid;

    if ($type)
      $where['msg_type'] = $type;

    if ($title)
      $where['msg_title'] = $title;

    return $wpdb->delete( $table_name, $where );
  }

  public  function delete_by_user( $user_id ) {
    $this->delete( 0, $user_id );
  }

  public function update_type( $id=0, $uid=0, $type='' ) {

    $id = intval($id);
    $uid = intval($uid);

    if ( ( !$id || !$uid) || empty($type) )
      return;

    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;

    if ( $id===0 ) {
      
      $result = $wpdb->update( $table_name, array(
        'msg_type' => $type
      ), array(
        'user_id' => $uid
      ) );

    } else {
      
      $result = $wpdb->update( $table_name, array(
        'msg_type' => $type
      ), array(
        'msg_id' => $id
      ) );

    }

    if ( ! $result )
      return 0;
    
    return 1;

  }

  public function get( $uid=0 , $count=0, $where='', $limit=0, $offset=0 ) {
    
    $uid = intval($uid);
    
    if ( !$uid )
      return;

    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;
    
    if ($count) {
      if($where) $where = " AND $where";
      $check = $wpdb->get_var( "SELECT COUNT(*) FROM $table_name WHERE user_id='$uid' $where" );
    } else {
      $check = $wpdb->get_results( "SELECT msg_id,msg_type,msg_date,msg_title,msg_content FROM $table_name WHERE user_id='$uid' AND $where ORDER BY msg_date DESC LIMIT $offset,$limit" );
    }
    
    if ($check)
      return $check;

    return 0;

  }

  public function get_credit_message( $uid=0 , $limit=0, $offset=0 ) {
    
    $uid = intval($uid);
    
    if ( !$uid )
      return;

    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;
    
    $check = $wpdb->get_results( "SELECT msg_id,msg_date,msg_title FROM $table_name WHERE msg_type='credit' AND user_id='$uid' ORDER BY msg_date DESC LIMIT $offset,$limit" );

    if ($check)
      return $check;

    return 0;

  }

  public function get_pm( $pm=0, $from=0, $count=false, $single=false, $limit=0, $offset=0 ) {
    
    $pm = absint($pm);
    $from = absint($from);

    if ( !$pm || !$from )
      return;
    
    $limit = absint($limit);
    $offset = absint($offset);

    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;

    $where_sql = " WHERE msg_type IN ('repm', 'unrepm')";

    if ($single) {

      $where_sql .= $wpdb->prepare( 
        " AND user_id = %d AND msg_title = %s",
        $pm,
        json_encode(array(
          'pm' => $pm,
          'from' => $from
        ))
      );
      
    } else {
      
      $where_sql .= $wpdb->prepare(
        " AND user_id IN (%d, %d) AND msg_title IN (%s, %s)",
        $pm,
        $from,
        json_encode(array(
          'pm' => $pm,
          'from' => $from
        )),
        json_encode(array(
          'pm' => $from,
          'from' => $pm
        ))
      );

    }

    if ($count) {
      $check = $wpdb->get_var( "SELECT COUNT(*) FROM $table_name $where_sql" );
    } else {
      $check = $wpdb->get_results( "SELECT msg_id,msg_date,msg_title,msg_content FROM $table_name $where_sql ORDER BY msg_date DESC LIMIT $offset,$limit" );
    }
    
    return $check ? $check : 0;
  }

  public function get_message_ajax_action_callback($pm_to, $paged) {
    $number = 10;
    $offset = ($paged-1)*$number;

    $current_user_id = get_current_user_id();

    $all = absint($this->get_pm($pm_to, $current_user_id, true));

    if ($all<1)
      return '<li>'.__('没有记录', 'dmeng').'</li>';

    $logs = $this->get_pm($pm_to, $current_user_id, false, false, $number, $offset);
    if (empty($logs))
      return '';
    
    $output = '';
    
    foreach ($logs as $log) {
      $pm_data = json_decode($log->msg_title);
			if( $pm_data->from==$pm_to ){
        $this->update_type($log->msg_id, $pm_to, 'repm');
			}
			$output .= '<li class="msg'.( $pm_data->from==$current_user_id ? ' highlight' : '' ).'"><div class="message-content clearfix"><span class="'.( $pm_data->from==$current_user_id ? 'pull-right' : 'pull-left' ).'">'.get_avatar( $pm_data->from, 34 ).'</span><div class="pm-box"><div class="pm-content">'.wpautop(wp_strip_all_tags(htmlspecialchars_decode($log->msg_content))).'</div><p class="pm-date">'.date_i18n( get_option( 'date_format' ).' '.get_option( 'time_format' ), strtotime($log->msg_date)).'</p></div></div></li>';
    }

    if ($paged<ceil($all/$number))
      $output .= '<li class="text-center"><a href="javascript:;" data-loading-text="'.__('加载中…', 'dmeng').'" id="load-more">'.__('加载更早的记录', 'dmeng').'</a></li>';
    
    return $output;
  }
  
  public function get_message_ajax_action() {

    do_action( 'dmeng_before_ajax' );
    
    if ( empty($_POST['user_id']) )
      die( '<li>'.__('用户ID不能为空', 'dmeng').'</li>' );
    
    $paged = max(1, (!empty($_POST['paged']) ? (int)$_POST['paged'] : 1)); 

    die( $this->get_message_ajax_action_callback(absint($_POST['user_id']), $paged) );
  }

  public function pm_ajax_action() {
    do_action( 'dmeng_before_ajax' );

    if ( empty($_POST['pm_to']) || empty($_POST['pm']) || wp_strip_all_tags(trim($_POST['pm']))=='' )
      die( __('收件人和私信内容都不能为空', 'dmeng') );

    $pm = wp_strip_all_tags(trim($_POST['pm']));
    $pm_to = absint($_POST['pm_to']);
    
    $user_info = get_userdata($pm_to);
    if (false===$user_info)
      die( __('收件用户不存在', 'dmeng') );

    global $current_user;
    get_currentuserinfo();
    
    $pm_check = $this->get_pm( $user_info->ID, $current_user->ID, false, true, 1 );
    if ( $pm_check && !empty($pm_check[0]) && !empty($pm_check[0]->msg_content) && $pm_check[0]->msg_content==$pm )
      die( __('请不要提交重复内容', 'dmeng') );
    
    if( $this->add(
          $user_info->ID, 
          'unrepm', 
          '', 
          json_encode(array(
            'pm' => $user_info->ID,
            'from' => $current_user->ID
          )), 
          $pm
        )
    ) {
				// 发邮件通知
				if (is_email($user_info->user_email)) {
					dmeng_mail(
            $user_info->user_email, 
            sprintf(__('%s给你发来了私信','dmeng'), $current_user->display_name), 
            '<h3>'.sprintf( __('%1$s，你好！','dmeng'), $user_info->display_name ).'</h3><p>'.sprintf( __('%1$s给你发来了<a href="%2$s" target="_blank">私信</a>，快去看看吧：<br> %3$s','dmeng'), $current_user->display_name, htmlspecialchars( add_query_arg('tab', 'message', get_author_posts_url( $user_info->ID )) ), wp_trim_words( $pm, 140, '…' )).'</p>'
          );
				}
        wp_send_json(array('success'=>1, 'result'=>$this->get_message_ajax_action_callback($user_info->ID, 1)));
    }
    die();
  }

}

$dmeng_Message = new dmeng_Message;
