<?php

/**
 * 
 */

class dmeng_Meta {
  
  public $db_version = 150921;
  
  public function __construct() {

    add_action( 'dmeng_db_setup', array( $this, 'db_setup' ) );
    if ( is_admin() && (int)get_option( $this->table_name.'_db_version' ) < $this->db_version ) {
      add_filter( 'dmeng_db_upgrade_check', '__return_true' );
      add_action( 'dmeng_db_upgrade', array( $this, 'db_upgrade' ) );
    }

    add_action( 'delete_user', array( $this, 'delete_user_action' ) );
  }
  
  /**
   * 数据库表格名称
   */
  public $table_name = 'dmeng_meta';   

  /**
   * 创建数据库表格
   */
  public function db_setup() {
    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;
    if ( $wpdb->get_var("SHOW TABLES LIKE '$table_name'")==$table_name )
      return false;

    $charset_collate = '';
    if ( ! empty($wpdb->charset) )
      $charset_collate = "DEFAULT CHARACTER SET $wpdb->charset";
    if ( ! empty($wpdb->collate) )
      $charset_collate .= " COLLATE $wpdb->collate";

    $result = $wpdb->query(
      "
        CREATE TABLE `$table_name` (
          `meta_id` bigint(20) unsigned NOT NULL AUTO_INCREMENT,
          `user_id` bigint(20) unsigned DEFAULT NULL,
          `meta_key` varchar(255) DEFAULT NULL,
          `meta_value` tinytext,
          PRIMARY KEY (`meta_id`),
          KEY `meta_key` (`meta_key`(191)),
          KEY `user_id` (`user_id`)
        ) $charset_collate;
      "
    );
    
    if (!$result)
      return false;

    update_option( $this->table_name.'_db_version', $this->db_version );
    return true;
  }

  public function db_upgrade() {
    
    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;
    
    if (true===$this->db_setup()) {
      echo '<p style="color:#090">'.sprintf(__('已成功创建 %s 数据表。', 'dmeng'), $table_name).'</p>';
      return;
    }

    $num = (int)$wpdb->get_var("SELECT count(*) FROM $table_name ");

    $index = array();
    $result = $wpdb->get_results("SHOW INDEX FROM $table_name");
    if ($result) {
      foreach ($result as $item) {
        if ('PRIMARY'!=$item->Key_name)
          $index[] = $item->Key_name;
      }
    }

    $upgrade_sql =
      "ALTER TABLE `$table_name` 
        ".(in_array('user_id', $index) ? 'DROP INDEX `user_id`,' : '')."
        ".(in_array('meta_key', $index) ? 'DROP INDEX `meta_key`,' : '')."
        MODIFY COLUMN `meta_id` bigint(20) UNSIGNED NOT NULL AUTO_INCREMENT FIRST,
        MODIFY COLUMN `user_id` bigint(20) UNSIGNED NULL AFTER `meta_id`,
        ADD INDEX `meta_key`(`meta_key`(191)),
        ADD INDEX `user_id`(`user_id`);
      ";

    if ($num>100000) {

      echo '<p style="color:#d54e21">'.sprintf(__('由于 %s 数据表过大，没有执行在线更新，建议手动更新。', 'dmeng'), $table_name).'</p>';
      
      echo '<p>'.__('MySQL更新代码可参考：', 'dmeng').'</p><pre><code>'.$upgrade_sql.'</code></pre>';
      
      return;
      
    } else {

      $result = $wpdb->query($upgrade_sql);
      
      if (!$result) {
        echo '<p style="color:#d54e21">'.sprintf(__('更新 %s 数据表失败，请重试。', 'dmeng'), $table_name).'</p>';
        return;
      }
      
      // 别忘了更新数据表版本号
      update_option( $this->table_name.'_db_version', $this->db_version );
      
      echo '<p style="color:#090">'.sprintf(__('已成功更新 %s 数据表。', 'dmeng'), $table_name).'</p>';
    }

  }
  
  /**
   * 计数
   */
  public function count( $key, $value=0, $uid='all' ) {

    $key = sanitize_text_field($key);
    $value = sanitize_text_field($value);
    if ( $uid!='all' )
      $uid = absint($uid);
    
    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;

    $sql = "SELECT count(meta_id) FROM $table_name WHERE meta_key LIKE '$key'";
    if ( $value )
      $sql .= " AND meta_value LIKE '$value'";
    if ( is_int($uid) )
      $sql .= " AND user_id = '$uid'";

    $check = $wpdb->get_var($sql);

    if ( isset($check) )
      return absint($check);
      
    return 0;
  }

  public function get_var( $key , $uid=0 ) {

    $key = sanitize_text_field($key);
    $uid = absint($uid);
    $cache_key = $key.'-'.$uid;
    
    if ($uid>0) {
      if (false !== ($cache = wp_cache_get($cache_key, $this->table_name)))
        return $cache;
    }
    
    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;
    
    $check = $wpdb->get_var( "SELECT meta_value FROM $table_name WHERE meta_key='$key' AND user_id='$uid' " );
    
    $result = isset($check) ? $check : 0;
    
    if ($uid>0) {
      wp_cache_set( $cache_key, $result, $this->table_name );
    }
    
    return $result;
  }

  public function get( $key , $uid=0, $limit=0, $offset=0 ) {

    $key = sanitize_text_field($key);
    $uid = absint($uid);
    
    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;
    
    $sql = "SELECT user_id, meta_value FROM $table_name WHERE meta_key='$key' ";
    if ($uid)
      $sql .= "AND user_id='$uid' ";
    
    $sql .= "ORDER BY meta_id DESC";
    
    if ($limit)
        $sql .= " LIMIT $offset,$limit";
    
    $check = $wpdb->get_results($sql);

    return isset($check) ? $check : 0;
  }

  public function exists($uid, $key, $value) {
    
    $key = sanitize_text_field($key);
    $value = sanitize_text_field($value);
    $uid = absint($uid);
    
    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;

    $meta_ids = $wpdb->get_col( $wpdb->prepare( "SELECT meta_id FROM $table_name WHERE meta_key = %s AND meta_value = %s AND user_id = %d ", $key, $value, $uid ) );
    
    return $meta_ids;

  }

  /**
   * 添加meta
   */
  public function add( $key, $value, $uid=0 ) {

    $key = sanitize_text_field($key);
    $value = sanitize_text_field($value);
    $uid = absint($uid);
    
    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;

    $result = $wpdb->insert( $table_name, array(
      'user_id' => $uid,
      'meta_key' => $key,
      'meta_value' => $value
    ) );
    
    if ( ! $result )
      return 0;
    
    if ($uid>0) {
      wp_cache_set( $key . '-' . $uid, $value, $this->table_name );
    }

    $mid = (int) $wpdb->insert_id;

    return $mid;
  }

  /**
   * 更新meta
   */
  public function update( $key, $value='', $uid=0 ) {

    $key = sanitize_text_field($key);
    $value = sanitize_text_field($value);
    $uid = absint($uid);
    $cache_key = $key . '-' . $uid;

    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;
    
    /**
     * 有用户ID的时候检查有没重复
     */
    if ( $uid>0 ) {
      $check = $wpdb->get_var( "SELECT meta_id FROM $table_name WHERE meta_key='$key' AND user_id='$uid' LIMIT 0,1 " );
    }
    
    if ( isset($check) ) {

      /**
       * 更新字段并更新缓存
       */

      $result = $wpdb->update( $table_name, array(
        'meta_value' => $value
      ), array(
        'meta_id' => $check
      ) );

      if ( ! $result )
        return false;

      wp_cache_replace( $cache_key, $value, $this->table_name );
      return $check;

    } else {

      /**
       * 插入字段并添加缓存
       */
       
      $result = $wpdb->insert( $table_name, array(
        'user_id' => $uid,
        'meta_key' => $key,
        'meta_value' => $value
      ) );

      if ( $uid>0 && $result ) {
        wp_cache_set( $cache_key, $value, $this->table_name );
        return 'inserted';
      }

    }

    return 0;
  }

  /**
   * 删除meta
   */
  public function delete( $key, $value=0, $uid='all' ) {

    $delete_all = 'all'==$uid;
    
    $key = sanitize_text_field($key);
    $uid = absint($uid);

    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;

    $where = array();
    $where['meta_key'] = $key;
    
    if ( $value )
      $where['meta_value'] = sanitize_text_field($value);

    if ( false===$delete_all )
      $where['user_id'] = $uid;

    $result = $wpdb->delete( $this->table_name, $where );

    if ( $result && $uid>0 ) {
      wp_cache_delete( $key.'-'.$uid, $this->table_name );
    }

    return $result;
  }

  public function delete_user_action( $user_id ) {

    global $wpdb;
    $table_name = $wpdb->prefix . $this->table_name;

    // 删除用户时，先删除非投票类数据
    $wpdb->query( " DELETE FROM $table_name WHERE user_id='$user_id' AND meta_key NOT LIKE 'vote_%' " );
    
    // 然后把剩余的，也就是投票数据的用户ID改为0（也就是游客）
    $wpdb->update( $table_name, array(
      'user_id' => 0
    ), array(
      'user_id' => $user_id
    ) );

  }

}

$dmeng_Meta = new dmeng_Meta;
